import os.path
from pathlib import Path
from typing import Union, List

from pydantic import BaseModel, Field

msg_xml_base = os.path.join(Path(__file__).parent, 'xml')


class WeixinReplyMsg(BaseModel):
    to_user: str = Field(title='接收方帐号（收到的OpenID）')
    from_user: str = Field(title='开发者微信号')
    create_time: int = Field(title='消息创建时间 （整型）')


class WeixinReplyTextMsg(WeixinReplyMsg):
    """文本消息"""
    _xml_file = os.path.join(msg_xml_base, 'text_msg.xml')
    content: str = Field(title='回复的消息内容',
                         description='回复的消息内容（换行：在 content 中能够换行，微信客户端就支持换行显示）')


class WeixinReplyImageMsg(WeixinReplyMsg):
    """图片消息"""
    _xml_file = os.path.join(msg_xml_base, 'image_msg.xml')
    media_id: str = Field(title='素材id', description='通过素材管理中的接口上传多媒体文件，得到的id。')


class WeixinReplyVoiceMsg(WeixinReplyMsg):
    """语音消息"""
    _xml_file = os.path.join(msg_xml_base, 'voice_msg.xml')
    media_id: str = Field(title='素材id', description='通过素材管理中的接口上传多媒体文件，得到的id。')


class WeixinReplyVideoMsg(WeixinReplyMsg):
    """视频消息"""
    _xml_file = os.path.join(msg_xml_base, 'video_msg.xml')
    media_id: str = Field(title='素材id', description='通过素材管理中的接口上传多媒体文件，得到的id。')
    title: Union[str, None] = Field(title='视频消息的标题')
    description: Union[str, None] = Field(title='视频消息的描述')


class WeixinReplyMusicMsg(WeixinReplyMsg):
    """音乐消息"""
    _xml_file = os.path.join(msg_xml_base, 'music_msg.xml')
    title: Union[str, None] = Field(title='音乐标题')
    description: Union[str, None] = Field(title='音乐描述')
    music_url: Union[str, None] = Field(title='音乐链接')
    hq_music_url: Union[str, None] = Field(title='高质量音乐链接', description='WIFI环境优先使用该链接播放音乐')
    thumb_media_id: str = Field(title='缩略图的媒体id', description='通过素材管理中的接口上传多媒体文件，得到的id')


class Article(BaseModel):
    _xml_file = os.path.join(msg_xml_base, 'news_msg_item.xml')
    title: str = Field(title='图文消息标题')
    description: str = Field(title='图文消息描述')
    pic_url: str = Field(title='图片链接', description='支持JPG、PNG格式，较好的效果为大图360*200，小图200*200')
    url: str = Field(title='点击图文消息跳转链接')


class WeixinReplyNewsMsg(WeixinReplyMsg):
    """图文消息"""
    _xml_file = os.path.join(msg_xml_base, 'news_msg.xml')
    article_count: int = Field(title='图文消息个数',
                               description='当用户发送文本、图片、语音、视频、图文、地理位置这六种消息时，开发者只能回复1条图文消息；其余场景最多可回复8条图文消息')
    articles: List[Article] = Field(title='图文消息信息', description='注意，如果图文数超过限制，则将只发限制内的条数')



import importlib
import os
from enum import Enum

from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.pydantic_tool.model.common_models import EnumItem


def auto_load_apps(root_path: str, root_name: str = 'apps',
                   app_info_name: str = 'app_info') -> dict[str, AppInfo]:
    """自动加载apps.{app_name}模块中的app_info"""
    app_dict = dict()
    for tmp_app_name in os.listdir(root_path):
        if tmp_app_name.startswith('__'):
            continue
        if os.path.isdir(os.path.join(root_path, tmp_app_name)):
            tmp_app = importlib.import_module(f'{root_name}.{tmp_app_name}')
            tmp_app_info = tmp_app.__getattribute__(app_info_name)
            if tmp_app_info:
                app_dict[tmp_app_info.code] = tmp_app_info
    return app_dict


def auto_set_enum_item_value(local_variable: dict):
    """自动设置Enum中EnumItem的value"""
    for tmp_key in {tmp for tmp in local_variable.keys() if tmp.endswith('Enum')}:
        tmp_value = local_variable.get(tmp_key)
        if isinstance(tmp_value, type) and issubclass(tmp_value, Enum):
            for tmp_enum in tmp_value:
                if isinstance(tmp_enum.value, EnumItem) and tmp_enum.value.value is None:
                    tmp_enum.value.value = tmp_enum.name



from urllib.parse import urljoin


def join_local_url(url_info_list: list[str | int]):
    """
    合并本地的url路径
    :param url_info_list: 本地url路径列表，如：['/static','/imags/','app-test', 'test.png']
    :return: /static/image/app-test/test.png
    """
    return '/' + '/'.join([str(tmp).removeprefix('/').removesuffix('/') for tmp in url_info_list if tmp is not None])


def join_url(base_url: str, url: str | int):
    """
    合并url地址
    :param base_url: 如：https://www.afengbook.com/book/100201
    :param url: 如：/book/100202
    :return:如：https://www.afengbook.com/book/100202
    """
    return urljoin(base_url, str(url))


if __name__ == '__main__':
    test_result = join_local_url(['/static', '/image/', 'app-test', 'test.png'])
    print(test_result)
    test_url_result = join_url('https://www.afengbook.com/book/100201', '/book/100202')
    print(test_url_result)



from afeng_tools.sqlalchemy_tools import sqlalchemy_settings


def get_base_model(db_code: str = 'default') -> type:
    """获取基础Model"""
    return sqlalchemy_settings.get_database(db_code).BaseModel


def is_model_instance(model_instance) -> bool:
    """是否是Model的实例"""
    if model_instance and not isinstance(model_instance, str) and not isinstance(model_instance, int) and not isinstance(model_instance, float) and not isinstance(model_instance, tuple) and not isinstance(model_instance, list) and not isinstance(model_instance, dict):
        return '_sa_instance_state' in model_instance.__dict__
    return False


def is_model_class(model_class: type) -> bool:
    """是否是Model的子类"""
    if model_class:
        return '__table__' in model_class.__dict__
    return False



"""
id和编码工具
"""


def get_code_by_id(id_value: int, begin_value=100000) -> int:
    """通过id获取编码"""
    return begin_value + id_value


def get_id_by_code(code: int, begin_value=100000) -> int:
    """通过编码获取id"""
    return code - begin_value


def get_tmp_pwd(code: int) -> str:
    """获取PWD"""
    if code <= 106603:
        return str(code - 999)[:4]
    return str(int(str(code)[::-1]) + code + 9999)[:4]



from typing import Literal, Optional, Any

from pydantic import BaseModel


class LayUiFormCascaderItem(BaseModel):
    """layui cascader 仿element-ui级联选择器: https://gitee.com/yixiacoco/lay_cascader """
    label: str
    value: str
    disabled: bool = False
    children: Optional[list['LayUiFormCascaderItem']] = None


class LayUiFormItem(BaseModel):
    """表单项"""
    title: str
    type: Literal['hidden', 'array',
    'text', 'number', 'email', 'radio', 'cascader', 'checkbox', 'switch', 'select', 'password', 'textarea', 'file'] = 'text'
    name: str
    autocomplete: Literal['off', 'on'] = 'off'
    default: Optional[str | int | Any] = None
    readonly: bool = False
    required: bool = False
    required_text: Optional[str] = None
    # 同时支持多条规则的验证("验证A|验证B"), required（必填项）phone（手机号） email（邮箱）url（网址）number（数字）date（日期）identity（身份证）自定义值
    verify: Optional[Literal['required', 'phone', 'email', 'url', 'number', 'date', 'identity'] | str] = None
    # 用于定义异常提示层模式。: tips（吸附层） alert（对话框） msg（默认）
    verify_type: Literal['tips', 'alert', 'msg'] = 'tips'
    placeholder: Optional[str] = None
    tip: Optional[str] = None
    # 当type为radio时， 下面 [(value, title, checked)]
    radio_list: Optional[list[tuple[str, str, bool]]] = None
    # 当type为cascader时， 下面 json.dumps([LayUiFormCascaderItem.model_dump()], ensure_ascii=False)
    cascader_list: Optional[str] = None
    # 当type为checkbox时， 下面 [(value, title, checked, disabled)]
    checkbox_list: Optional[list[tuple[str, str, bool, bool]]] = None
    # 当type为switch时， 下面 (value(0/1), title(开启|关闭 / ON|OFF), checked, disabled)
    switch_item: Optional[tuple[str, str, bool, bool]] = None
    # 当type为select时， 下面 [(value, title, selected, disabled)]
    select_list: Optional[list[tuple[str, str, bool, bool]]] = None
    # 当type为file时
    file_upload_url: Optional[str] = None
    file_upload_title: Optional[str] = '上传文件'
    # 上传属性（上传时需要的额外表单参数）
    file_upload_params: Optional[list[str]] = None
    # 上传后的处理操作
    file_upload_after_handle_js: Optional[str] = None


class LayUiTableCol(BaseModel):
    """表格列"""
    field: str
    title: str
    width: Optional[int] = None
    min_width: Optional[int] = None
    sort: bool = False
    is_enum: bool = False


class LayUiTableData(BaseModel):
    """表格数据"""
    code: int = 0
    message: Optional[str] = None
    total: int
    data_list: list


class LayUiTableActionItem(BaseModel):
    """表格操作"""
    # 标题
    title: str
    url: str
    # 完成后是否刷新表格页面
    reload: bool = False
    # 参数连接字符
    param_link_char: str = '?'
    param_name: str = 'id_value'
    param_field: str = 'id'
    target: Literal['_self', '_parent', '_blank'] = '_self'


class LayUiFormStep(BaseModel):
    code: str
    # 步骤标题
    title: str
    # 步骤表单
    form_item_list: list[LayUiFormItem]
    # 提交链接
    submit_url: str
    after_handle_js: Optional[str] = None



"""
文件工具
"""
import glob
import os
import re
import shutil
from typing import Union

from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.linux_tool import linux_match_tools
from afeng_tools.math_tool import random_tools


def list_path(dir_path, include_file_list=None, exclude_dir_list: list = None,
              exclude_file_list: list = None):
    """
    列出目录下的指定文件和目录
    :param dir_path: 目录路径
    :param include_file_list: 文章后缀名集合（可以用正则表达式）
    :param exclude_dir_list: 排除的子目录
    :param exclude_file_list: 排除的文件，可以用正则表达式
    :return: {‘文件名’:'文件完整路径'}
    """
    if include_file_list is None:
        include_file_list = ['*.*']
    exclude_dir_list = [] if exclude_dir_list is None else exclude_dir_list
    exclude_file_list = [] if exclude_file_list is None else exclude_file_list
    file_dict = dict()
    for tmp_file in os.listdir(dir_path):
        tmp_file_full_path = os.path.join(dir_path, tmp_file)
        if os.path.isfile(tmp_file_full_path) \
                and linux_match_tools.match_any(tmp_file, include_file_list) \
                and not linux_match_tools.match_any(tmp_file, exclude_file_list):
            tmp_file_name = tmp_file.rsplit('.', 1)[0]
            file_dict[tmp_file_name] = tmp_file_full_path
        if os.path.isdir(tmp_file_full_path) \
                and not linux_match_tools.match_any(tmp_file, exclude_dir_list):
            file_dict[tmp_file] = tmp_file_full_path
    return file_dict


def get_file_list(dir_path, filter_pattern='*.*', recursive=False):
    """
    列出文件列表
    :param dir_path: 文件路径
    :param filter_pattern: 文件名过滤模式，如：*.png
    :param recursive: 如果为True，会遍历子文件夹
    :return:
    """
    return glob.glob(os.path.join(dir_path, filter_pattern), recursive=recursive)


def get_image_list(dir_path):
    """
    获取文件夹中所有图像文件的列表
    :param dir_path: 文件路径
    :return:
    """
    return get_file_list(dir_path, '*.png')


def _sort_key_lambda(x: str) -> str:
    """
    排序键的lambda表达式
    :param x: 键x
    :return: 改动后的比较键x
    """
    if '\\' in x:
        x = x.rsplit('\\', 1)[1]
    if '/' in x:
        x = x.rsplit('/', 1)[1]
    re_search = re.search('(\d*)', x)
    if re_search:
        return str(-10000000000000 - int(re_search.group(1)))
    return str.lower(x)


def filename_sort(filename_list: list, reverse=False):
    """
    文件名排序排序
    :param filename_list: 文件名列表
    :param reverse: 是否反转
    :return: 排序后的列表（直接在原列表排序的）
    """

    filename_list.sort(
        key=lambda x: _sort_key_lambda(x),
        reverse=reverse)
    return filename_list


def random_file_name():
    """返回由时间戳+5位随机数的文件名"""
    return random_tools.random_str()


def read_file(data_file, binary_flag=False) -> Union[str, bytes]:
    """
    读取文件字符串
    :param data_file: 数据文件
    :param binary_flag: 是否是二进制文件，如果True,则按照二进制读取
    :return: None或文件内容
    """
    if os.path.isfile(data_file):
        mode = 'rb' if binary_flag else 'r'
        encoding = None if binary_flag else 'utf-8'
        with open(data_file, mode, encoding=encoding) as f:
            return f.read()


def iterate_read_file(data_file: str, binary_flag=False, buffer_size=50 * 1024):
    """
    递归读取文件字符串
    :param data_file: 数据文件
    :param binary_flag: 是否是二进制文件，如果True,则按照二进制读取
    :param buffer_size: 缓存区大小
    :return: None或文件内容
    """
    if os.path.isfile(data_file):
        mode = 'rb' if binary_flag else 'r'
        encoding = None if binary_flag else 'utf-8'
        with open(data_file, mode, encoding=encoding) as f:
            # 读取buffer_size字节的数据
            data = f.read(buffer_size)
            while data:
                # 处理数据
                yield data
                # 继续读取下一个buffer_size字节的数据
                data = f.read(buffer_size)


def save_file(file_data: str | bytes, save_file_name: str, binary_flag=False) -> str:
    """
    保存文件
    :param file_data: 文件内容
    :param save_file_name: 保存的文件
    :param binary_flag: 是否是二进制文件，如果True,则按照二进制读取
    :return: 保存的文件
    """
    mode = 'wb' if binary_flag else 'w'
    encoding = None if binary_flag else 'utf-8'
    with open(save_file_name, mode, encoding=encoding) as f:
        f.write(file_data)
    return save_file_name


def save_html_file(save_html_name: str, file_data: str | bytes, append_jinja2_raw=False):
    """
    保存html文件
    :param save_html_name: 保存的html文件
    :param file_data: 文件内容
    :param append_jinja2_raw: 添加jinja2的{% raw %}， 用于确保模板中的某些内容不会被错误地转义
    :return: 保存的html文件
    """
    binary_flag = isinstance(file_data, bytes)
    mode = 'wb' if binary_flag else 'w'
    encoding = None if binary_flag else 'utf-8'
    if append_jinja2_raw:
        if binary_flag:
            file_data = '{% raw %}'.encode('utf-8') + file_data + '{% endraw %}'.encode('utf-8')
        else:
            file_data = '{% raw %}' + file_data + '{% endraw %}'
    with open(save_html_name, mode, encoding=encoding) as f:
        f.write(file_data)
    return save_html_file


def append_file(file_data: str | bytes, save_file_name: str, binary_flag=False) -> str:
    """
    保存文件
    :param file_data: 文件内容
    :param save_file_name: 保存的文件
    :param binary_flag: 是否是二进制文件，如果True,则按照二进制读取
    :return: 保存的文件
    """
    mode = 'ab' if binary_flag else 'a'
    encoding = None if binary_flag else 'utf-8'
    with open(save_file_name, mode, encoding=encoding) as f:
        f.write(file_data)
    return save_file_name


def read_files(file_path: str, file_pattern: str) -> dict[str, str]:
    """
    读取某些文件的内容
    :param file_path: 文件路径
    :param file_pattern: 文件名规则， 如: *。css
    :return: {'文件名','文件内容'}
    """
    tmp_file_list = get_file_list(file_path, file_pattern)
    tmp_file_list = filename_sort(tmp_file_list)
    file_info_dict = {}
    for tmp_file in tmp_file_list:
        file_info_dict[os.path.split(tmp_file)[1]] = read_file(tmp_file)
    return file_info_dict


def read_files_to_one(file_path: str, file_pattern: str, join_char: str = '\n') -> str:
    """
    读取某些文件为一个字符串
    :param file_path: 文件路径
    :param file_pattern: 文件名规则， 如: *。css
    :param join_char: 文件内容拼接的字符
    :return: 文件内容拼接的字符串
    """
    file_info_dict = read_files(file_path=file_path, file_pattern=file_pattern)
    tmp_content_list = []
    for _, tmp_content in file_info_dict.items():
        tmp_content_list.append(tmp_content)
    return join_char.join(tmp_content_list)


def read_file_lines(data_file) -> list[str]:
    """
    读取文件行（空白行已经清除）
    :param data_file: 数据文件
    :return: 行数据组成的列表
    """
    with open(data_file, encoding='utf-8') as file:
        line_list = []
        for tmp_line in file.readlines():
            tmp_line = tmp_line.strip()
            if tmp_line:
                line_list.append(tmp_line)
        return line_list


def calc_new_file_name(old_file: str):
    """计算新的文件名"""
    old_file_name = old_file.split('/')[-1]
    subfix = old_file_name.rsplit('.', maxsplit=1)[-1]
    return hashlib_tools.calc_md5(old_file_name) + '.' + subfix


def list_files(dir_path: str, recursion: bool = True, include_folder: bool = True) -> tuple[list[str], list[str]]:
    """
    列出目录下所有文件
    :param dir_path: 目录
    :param recursion: 是否递归遍历子目录，默认是
    :param include_folder: 是否包含目录，默认是
    :return: (相对所有子文件、子文件夹列表，绝对所有子文件、子文件夹列表)
    """
    # 相对文件列表
    relative_file_list = []
    for tmp_dir in os.listdir(dir_path):
        abs_dir = os.path.join(dir_path, tmp_dir)
        if os.path.isdir(abs_dir):
            if include_folder:
                relative_file_list.append(tmp_dir)
            if recursion:
                tmp_relative, tmp_absolute = list_files(abs_dir, recursion=recursion, include_folder=include_folder)
                relative_file_list.extend([os.path.join(tmp_dir, temp) for temp in tmp_relative])
        else:
            relative_file_list.append(tmp_dir)
    # 绝对文件列表
    absolute_file_list = [os.path.join(dir_path, tmp_dir) for tmp_dir in relative_file_list]
    return relative_file_list, absolute_file_list


def read_file_body(data_file, start: int, length: int) -> bytes:
    """
    读取文件内容
    :param data_file: 数据内容
    :param start: 开始位置
    :param length: 读取长度
    :return: 读取到的字节码
    """
    with open(data_file, 'rb') as tmp_file:
        tmp_file.seek(start)
        return tmp_file.read(length)


one_kb = 1024
one_mb = 1024 * 1024
one_gb = 1024 * 1024 * 1024


def calc_file_size(file_size: int) -> str | None:
    """计算文件大小"""
    if file_size is None:
        return file_size
    if file_size < 1024:
        return f'{file_size}B'
    elif file_size // one_kb < 1024:
        return f'{round(file_size / one_kb, 2)}KB'
    elif file_size // one_mb < 1024:
        return f'{round(file_size / one_mb, 2)}MB'
    else:
        return f'{round(file_size / one_gb, 2)}GB'


def parse_file_size(file_size: str) -> int | None:
    """
    转换文件大小
    :param file_size: 文件大小，如：2MB
    :return: 文件大小int值
    """
    if file_size is None:
        return file_size
    file_size = file_size.strip()
    if file_size.endswith('KB'):
        return int(float(file_size.removesuffix('KB')) * one_kb)
    elif file_size.endswith('K'):
        return int(float(file_size.removesuffix('K')) * one_kb)
    elif file_size.endswith('MB'):
        return int(float(file_size.removesuffix('MB')) * one_mb)
    elif file_size.endswith('M'):
        return int(float(file_size.removesuffix('M')) * one_mb)
    elif file_size.endswith('GB'):
        return int(float(file_size.removesuffix('GB')) * one_gb)
    elif file_size.endswith('G'):
        return int(float(file_size.removesuffix('G')) * one_gb)
    elif file_size.endswith('B'):
        return int(file_size.removesuffix('B'))
    return int(file_size)


def copy_directory(source: str, destination: str, delete_old: bool = True, include_dir_name: bool = False):
    """
    复制文件夹
    :param source: 源目录
    :param destination: 目标目录
    :param delete_old: 是否删除旧的目标目录
    :param include_dir_name: 目标目录中是否包含复制后的目录名
    :return:
    """
    if not include_dir_name:
        destination = os.path.join(destination, os.path.split(source)[1])
    if delete_old and os.path.exists(destination):
        # 如果目标文件夹已存在，删除它
        shutil.rmtree(destination)
    # 复制源文件夹到目标文件夹
    shutil.copytree(source, destination)


if __name__ == '__main__':
    # test_path = 'C:\\Users\\chentiefeng\\AppData\\Local\\JianyingPro\\User Data'
    # file_list = get_file_list(test_path, '*.mp3', True)
    # for tmp in file_list:
    #     print(tmp)

    book_path = r'C:\迅雷云盘'
    relative_list, absolute_list = list_files(book_path)
    for tmp in relative_list:
        print(tmp)
    for tmp in absolute_list:
        print(tmp)



from typing import Any

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class ArticlePoService(PoService):
    """
    使用示例：article_po_service = ArticlePoService(app_info.db_code, ArticleInfoPo)
    """
    _table_name_ = "tb_article_info"

    def query_by_code(self, type_code: str, article_code: str) -> Any:
        return self.get(self.model_type.type_code == type_code, self.model_type.code == article_code)




"""
Fast API App工具集
"""

from fastapi import FastAPI, APIRouter
from starlette.middleware.cors import CORSMiddleware
from starlette.middleware.gzip import GZipMiddleware
from starlette.staticfiles import StaticFiles

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.application_models import AppInfo
from afeng_tools.application_tool.settings_enum import SettingsStaticItem, SettingsKeyEnum
from afeng_tools.fastapi_tool.core.fastapi_exception_handler import register_exception_handler


def _register_cors(app: FastAPI):
    """跨域设置"""
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )


def _register_router(app: FastAPI, router_list: list[APIRouter] = None, router_dict: dict[str, APIRouter] = None):
    """注册路由"""
    # 注册router
    if router_list:
        for tmp_router in router_list:
            app.include_router(tmp_router)
    if router_dict:
        for tmp_prefix, tmp_router in router_dict.items():
            app.include_router(tmp_router, prefix=tmp_prefix)


def _register_middleware(app: FastAPI):
    """注册中间件"""
    # minimum_size：需要 GZip 响应的最小大小（以字节为单位）。默认值为 500。
    app.add_middleware(GZipMiddleware, minimum_size=1000)


def _register_static_file(app: FastAPI):
    """注册静态文件"""
    # 静态文件
    app_static_value = settings_tools.get_config(SettingsKeyEnum.server_static)
    if isinstance(app_static_value, list):
        for i, tmp_static in enumerate(app_static_value):
            if isinstance(tmp_static, SettingsStaticItem):
                app.mount(path=tmp_static.url,
                          app=StaticFiles(directory=tmp_static.path, html=True),
                          name=tmp_static.name if tmp_static.name else f'static_{i}')
    else:
        if isinstance(app_static_value, SettingsStaticItem):
            app.mount(path=app_static_value.url,
                      app=StaticFiles(directory=app_static_value.path, html=True),
                      name=app_static_value.name if app_static_value.name else f'static')


def create_app(router_list: list[APIRouter] = None, router_dict: dict[str, APIRouter] = None,
               app_dict: dict[str, AppInfo] = None):
    """创建FastAPI应用"""
    is_debug = settings_tools.get_config(SettingsKeyEnum.app_is_debug)
    app = FastAPI(debug=is_debug, openapi_url=None, docs_url=None, redoc_url=None)
    # 注册路由
    _register_router(app, router_list=router_list, router_dict=router_dict)
    # 注册捕获全局异常
    register_exception_handler(app, app_dict=app_dict)
    # 注册中间件（请求拦截）
    _register_middleware(app)
    # 跨域设置
    _register_cors(app)
    # 注册静态资源
    _register_static_file(app)
    return app



"""
csv工具：
    - pandas： pip install pandas
"""
import pandas
from pandas import DataFrame


def read(csv_file: str, encoding: str = 'utf-8', sep: str = ',',
         n_rows: int = None, use_cols: list = None) -> DataFrame:
    """
    读取csv文件
    :param csv_file: csv文件
    :param encoding: 编码，默认是 utf-8
    :param sep: 指明分隔符，默认是逗号
    :param n_rows: 指明读取行数,如：2（读取0、1、2行）
    :param use_cols: 读取哪些列，如：[1,3] 读取第1、3列
    :return:
    """
    return pandas.read_csv(csv_file, encoding=encoding, sep=sep, nrows=n_rows, usecols=use_cols, dtype=str,
                           engine='python')


if __name__ == '__main__':
    csv = r'C:\Users\chentiefeng\Downloads\tushare_stock_basic_20230907151542.csv'
    df = read(csv)
    row_count, col_count = df.shape
    for row_i in range(row_count):
        for col_i in range(col_count):
            cell = df.iloc[row_i, col_i]
            print(cell, end=',')
        print()



"""
jinja2自定义过滤器
"""
import re

from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.jinja2_tool.jinja2_decorator_tools import jinja2_filter


@jinja2_filter('value')
def filter_none_value(value, default: int | float | str | list | dict = None,
                      is_list: bool = False, is_obj: bool = False):
    if value is None:
        if is_list:
            return default if default is not None else []
        elif is_obj:
            return default if default is not None else dict()
        else:
            return default if default is not None else ''
    return value


@jinja2_filter('md5')
def filter_md5(value):
    if value is None:
        return ''
    result_md5 = hashlib_tools.calc_md5(value)
    re_match = re.match('(\d*).*?', result_md5)
    if re_match:
        result_md5 = result_md5.removeprefix(re_match.group(1))
    return result_md5


if __name__ == '__main__':
    print(filter_md5('测试啊'))



"""
阿里云盘分享工具 pip install aligo -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import os
from typing import List, Callable

from aligo import Aligo, BaseShareFile, GetShareTokenResponse, ShareLinkExtractCodeResponse, \
    BatchShareFileSaveToDriveResponse

from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.file_tool.tmp_file_tools import get_user_tmp_dir
from afeng_tools.serialization_tool import pickle_tools


def get_share_info_by_link(ali_api: Aligo, share_msg: str) -> ShareLinkExtractCodeResponse:
    """
    通过链接获取分享信息
    :param ali_api: Aligo
    :param share_msg: 分享信息
                如：「心灵奇旅 4K 原盘 REMUX」https://www.aliyundrive.com/s/FYoddEbsSwV
                如： https://www.aliyundrive.com/s/LkUS6BrconT/folder/62074d95b39e519f60be4f73b68ac493e9977a90
    :return: 分享信息，如：result.share_id, result.share_pwd
    """
    return ali_api.share_link_extract_code(share_msg)


def get_share_token(ali_api: Aligo, share_id: str, share_pwd: str = '') -> GetShareTokenResponse:
    """获取share_token"""
    return ali_api.get_share_token(share_id, share_pwd=share_pwd)


def list_share_root_files(ali_api: Aligo, share_token: GetShareTokenResponse) -> List[BaseShareFile]:
    """
    列出分享根路径中的文件
    :param ali_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :return: List[BaseShareFile]
    """
    return ali_api.get_share_file_list(share_token)


def list_share_path_files(ali_api: Aligo, share_token: GetShareTokenResponse,
                          parent_file_id='root') -> List[BaseShareFile]:
    """
    列出分享根路径中的文件
    :param ali_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :param parent_file_id: parent_file_id
    :return: List[BaseShareFile]
    """
    return ali_api.get_share_file_list(share_token, parent_file_id=parent_file_id)


def _tree_share(ali_api: Aligo, share_token: GetShareTokenResponse,
                parent_file_id: str = 'root', parent_path: str = None,
                root_cache_path: str = None,
                callback_func: Callable = None) -> List[tuple[str, BaseShareFile]]:
    """
    树形递归获取分享文件
    :param ali_api:  Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :param parent_file_id: parent_file_id
    :param parent_path: parent_path
    :param root_cache_path: 当是内部递归调用时，传入缓存的根路径
    :param callback_func: 获取到文件信息后的回调函数, 有两个参数：pan_path:str, pan_file:BaseShareFile
    :return: [(路径,BaseShareFile)]
    """
    try:
        file_list = ali_api.get_share_file_list(share_token, parent_file_id=parent_file_id)
    except KeyError as e:
        share_token = get_share_token(ali_api, share_token.share_id, share_token.share_pwd)
        file_list = ali_api.get_share_file_list(share_token, parent_file_id=parent_file_id)
    if file_list:
        all_files = []
        is_root: bool = False
        if root_cache_path is None:
            is_root = True
            root_cache_path = os.path.join(get_user_tmp_dir(), hashlib_tools.calc_md5(share_token.share_id))
            os.makedirs(root_cache_path, exist_ok=True)
        for tmp_file in file_list:
            pan_path = parent_path + '/' + tmp_file.name if parent_path else tmp_file.name
            print(f'[{tmp_file.type}]{pan_path}')
            if tmp_file.type == 'folder':
                tmp_cache = os.path.join(root_cache_path, tmp_file.file_id)
                if os.path.exists(tmp_cache):
                    child_list = pickle_tools.parse_to_obj(tmp_cache)
                    if child_list and callback_func and isinstance(callback_func, Callable):
                        for temp_child_file_name, temp_child_file in child_list:
                            callback_func(temp_child_file_name, temp_child_file)
                else:
                    child_list = _tree_share(ali_api, share_token,
                                             parent_file_id=tmp_file.file_id,
                                             parent_path=pan_path, root_cache_path=root_cache_path,
                                             callback_func=callback_func)
                if child_list:
                    all_files.extend(child_list)
                    if not os.path.exists(tmp_cache):
                        pickle_tools.save_to_file(child_list, tmp_cache)
            else:
                all_files.append((pan_path, tmp_file))
                if callback_func and isinstance(callback_func, Callable):
                    if callback_func and isinstance(callback_func, Callable):
                        callback_func(pan_path, tmp_file)
        if is_root:
            # 先注释了删除
            # shutil.rmtree(root_cache_path)
            pass
        return all_files


def list_share_all_file(ali_api: Aligo, share_token: GetShareTokenResponse,
                        parent_file_id='root', parent_path: str = None,
                        callback_func: Callable = None) -> List[tuple[str, BaseShareFile]]:
    """
    递归遍历文件
    :param ali_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :param parent_file_id: 父 file_id
    :param parent_path: 父路径
    :param callback_func: 获取到文件信息后的回调函数, 有两个参数：pan_path:str, pan_file:BaseShareFile
    :return: [(路径,BaseShareFile)]
    """
    return _tree_share(ali_api, share_token, parent_file_id=parent_file_id, parent_path=parent_path,
                       callback_func=callback_func)


def save_share_file_to_pan(ali_api: Aligo,
                           share_token: GetShareTokenResponse) -> List[BatchShareFileSaveToDriveResponse]:
    """
    保存所有分享文件到云盘
    :param ali_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :return: List[BatchShareFileSaveToDriveResponse]
    """
    # 如果遇到分享文件非常多，此段代码运行完成后，不会立马看到所有文件，可能需要几个小时才能陆续保存完成
    # 在网页保存需要几个小时，用这个一下就可以了，阿里云服务器会处理，不用等待
    return ali_api.share_file_save_all_to_drive(share_token)


def download_share_file(ali_api: Aligo, share_token: GetShareTokenResponse, local_save_path: str,
                        download_before_callback: Callable[[str, BaseShareFile], bool] = None,
                        download_after_callback: Callable[[str, BaseShareFile, str], bool] = None):
    """
    下载分享中的文件到本地
    :param ali_api: Aligo
    :param share_token: 可以调用 get_share_token() 方法获取
    :param local_save_path: 本地存储路径
    :param download_before_callback: lambda pan_path,share_file: True
    :param download_after_callback: lambda pan_path,share_file,local_file: True
    :return:
    """
    for tmp_pan_path, tmp_file in list_share_all_file(ali_api, share_token):
        download_flag = True
        if download_before_callback and isinstance(download_before_callback, Callable):
            download_flag = download_before_callback(tmp_pan_path, tmp_file)
        if download_flag:
            download_file = ali_api.get_share_link_download_url(tmp_file.file_id, share_token)
            local_file = ali_api.download_file(file_path=local_save_path,
                                               url=download_file.download_url or download_file.url)
            if download_after_callback and isinstance(download_after_callback, Callable):
                download_after_callback(tmp_pan_path, tmp_file, local_file)


def cancel_share(ali_api: Aligo, share_id: str):
    """取消分享"""
    ali_api.cancel_share(share_id)


def create_aligo_share(ali_api: Aligo, file_id: str) -> str:
    """创建Aligo分享"""
    return ali_api.share_folder_by_aligo(file_id)


def save_aligo_share(ali_api: Aligo, aligo_share_data: str) -> list:
    """保存Aligo分享"""
    return ali_api.save_files_by_aligo(aligo_share_data)



"""
百度配置
"""
import os.path
import tempfile
from afeng_tools.os_tool.os_tools import get_user_home
from afeng_tools.baidu_pan_tool.baidu_pan_enum import BaidupanConfigKeyEnum

__BAIDU_PAN_CONFIG_CACHE__ = {
    BaidupanConfigKeyEnum.app_name: 'www',
    BaidupanConfigKeyEnum.app_id: 'xxx',
    BaidupanConfigKeyEnum.app_key: 'xxxx',
    BaidupanConfigKeyEnum.secret_key: 'xxx',
    BaidupanConfigKeyEnum.sign_key: 'xxx',
    BaidupanConfigKeyEnum.redirect_url: 'oob',
    # 认证二维码存储位置
    BaidupanConfigKeyEnum.auth_qrcode_image: os.path.join(tempfile.gettempdir(), 'baidu_auth_qrcode.png'),
    # token文件路径
    BaidupanConfigKeyEnum.token_file: os.path.join(get_user_home(), '.baidu_token.bin'),
    # 百度网盘存储根路径
    BaidupanConfigKeyEnum.pan_root_path: '/apps/website'
}

from afeng_tools.os_tool.os_tools import get_user_home


def init_app_config(app_id: str, app_key: str, secret_key: str, sign_key: str):
    """设置应用配置配置"""
    _set_config(BaidupanConfigKeyEnum.app_id, app_id)
    _set_config(BaidupanConfigKeyEnum.app_key, app_key)
    _set_config(BaidupanConfigKeyEnum.secret_key, secret_key)
    _set_config(BaidupanConfigKeyEnum.sign_key, sign_key)


def _set_config(config_key: BaidupanConfigKeyEnum, config_value: str):
    """设置配置"""
    __BAIDU_PAN_CONFIG_CACHE__[config_key] = config_value


def get_config(config_key: BaidupanConfigKeyEnum) -> str:
    """获取配置"""
    return __BAIDU_PAN_CONFIG_CACHE__.get(config_key)



from typing import Any

from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class GroupPoService(PoService):
    """
    使用示例：group_po_service = GroupPoService(app_info.db_code, GroupInfoPo)
    """
    _table_name_ = "tb_group_info"

    def get_group(self, title: str, app_code: str, type_code: str) -> Any:
        group_po = self.get(self.model_type.title == title,
                            self.model_type.app_code == app_code, self.model_type.type_code == type_code)
        if not group_po:
            group_po = self.save(self.model_type(
                title=title,
                app_code=app_code,
                type_code=type_code
            ), auto_code=True)
        return group_po



import openapi_client
from openapi_client.api import auth_api, userinfo_api, multimediafile_api, filemanager_api, fileinfo_api, fileupload_api
from afeng_tools.decorator_tool.decorator_tools import run_func


def auto_api_client(func):
    """自动导入 api_client:ApiClient"""
    def wrap(*args, **kwargs):
        if 'api_client' in kwargs:
            return run_func(func, *args, **kwargs)
        with openapi_client.ApiClient() as api_client:
            kwargs['api_client'] = api_client
            return run_func(func, *args, **kwargs)
    return wrap


def auto_auth_api(func):
    """自动导入 api_instance:AuthApi"""
    def wrap(*args, **kwargs):
        if 'api_instance' in kwargs:
            return run_func(func, *args, **kwargs)
        with openapi_client.ApiClient() as api_client:
            api_instance = auth_api.AuthApi(api_client)
            kwargs['api_instance'] = api_instance
            return run_func(func, *args, **kwargs)
    return wrap


def auto_userinfo_api(func):
    """自动导入 api_instance:UserinfoApi"""
    def wrap(*args, **kwargs):
        if 'api_instance' in kwargs:
            return run_func(func, *args, **kwargs)
        with openapi_client.ApiClient() as api_client:
            api_instance = userinfo_api.UserinfoApi(api_client)
            kwargs['api_instance'] = api_instance
            return run_func(func, *args, **kwargs)
    return wrap


def auto_fileinfo_api(func):
    """自动导入 api_instance:FileinfoApi"""
    def wrap(*args, **kwargs):
        if 'api_instance' in kwargs:
            return run_func(func, *args, **kwargs)
        with openapi_client.ApiClient() as api_client:
            api_instance = fileinfo_api.FileinfoApi(api_client)
            kwargs['api_instance'] = api_instance
            return run_func(func, *args, **kwargs)
    return wrap


def auto_media_file_api(func):
    """自动导入 api_instance:MultimediafileApi"""
    def wrap(*args, **kwargs):
        if 'api_instance' in kwargs:
            return run_func(func, *args, **kwargs)
        with openapi_client.ApiClient() as api_client:
            api_instance = multimediafile_api.MultimediafileApi(api_client)
            kwargs['api_instance'] = api_instance
            return run_func(func, *args, **kwargs)
    return wrap


def auto_file_manager_api(func):
    """自动导入 api_instance:FilemanagerApi"""
    def wrap(*args, **kwargs):
        if 'api_instance' in kwargs:
            return run_func(func, *args, **kwargs)
        with openapi_client.ApiClient() as api_client:
            api_instance = filemanager_api.FilemanagerApi(api_client)
            kwargs['api_instance'] = api_instance
            return run_func(func, *args, **kwargs)
    return wrap


def auto_file_upload_api(func):
    """自动导入 api_instance:FileuploadApi"""
    def wrap(*args, **kwargs):
        if 'api_instance' in kwargs:
            return run_func(func, *args, **kwargs)
        with openapi_client.ApiClient() as api_client:
            api_instance = fileupload_api.FileuploadApi(api_client)
            kwargs['api_instance'] = api_instance
            return run_func(func, *args, **kwargs)
    return wrap





"""
LayUI工具
- pip install pydantic -i https://pypi.tuna.tsinghua.edu.cn/simple/
- pip install Jinja2 -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""

import os.path
from typing import Callable

from afeng_tools.jinja2_tool import jinja2_tools
from afeng_tools.layui_tool.core.layui_models import LayUiFormItem, LayUiTableCol, LayUiTableData, LayUiTableActionItem, \
    LayUiFormStep

template_env = jinja2_tools.create_template_env([os.path.join(os.path.dirname(__file__), 'template')])


def table_data_parse(data_list: list, parse_callback: Callable[[list], LayUiTableData]) -> LayUiTableData:
    """表单数据转换"""
    return parse_callback(data_list)


def render_step_form_template(title: str, form_step_list: list[LayUiFormStep], height: int = 500,
                              file_upload_url: str = '/admin/resource/upload') -> str:
    return jinja2_tools.format_template('form/step_form.html', context={
        'title': title,
        'height': height,
        'form_step_list': form_step_list,
        'file_upload_url': file_upload_url
    }, env=template_env)


def render_add_template(title: str, form_item_list: list[LayUiFormItem], add_url: str,
                        has_parent: bool = True, finish_redirect_url: str = None,
                        file_upload_url: str = '/admin/resource/upload') -> str:
    return jinja2_tools.format_template('table/add.html', context={
        'title': title,
        'item_list': form_item_list,
        'add_url': add_url,
        'has_parent': has_parent,
        'finish_redirect_url': finish_redirect_url,
        'file_upload_url': file_upload_url
    }, env=template_env)


def render_edit_template(title: str, form_item_list: list[LayUiFormItem], edit_url: str,
                         file_upload_url: str = '/admin/resource/upload') -> str:
    return jinja2_tools.format_template('table/edit.html', context={
        'title': title,
        'item_list': form_item_list,
        'edit_url': edit_url,
        'file_upload_url': file_upload_url
    }, env=template_env)


def render_show_template(title: str, form_item_list: list[LayUiFormItem]) -> str:
    return jinja2_tools.format_template('table/show.html', context={
        'title': title,
        'item_list': form_item_list,
    }, env=template_env)


def render_table_template(title: str, search_form_item_list: list[LayUiFormItem], data_url: str,
                          col_list: list[LayUiTableCol],
                          page_name: str = 'page', limit_name: str = 'limit',
                          search_params_name: str = 'searchParams',
                          add: bool = False, add_html_url: str = 'add',
                          edit: bool = False, edit_html_url: str = 'edit',
                          delete: bool = False, delete_url: str = None, delete_field: str = 'id',
                          more_delete: bool = False, more_delete_url: str = None, more_delete_name: str = 'ids',
                          more_delete_field: str = 'id',
                          action_list: list[LayUiTableActionItem] = None,
                          message: str = None) -> str:
    """
    渲染表格模板
    :param title:
    :param search_form_item_list:
    :param data_url:
    :param col_list:
    :param page_name: 页码的参数名称，默认：page
    :param limit_name: 每页数据量的参数名，默认：limit
    :param search_params_name:
    :param add:
    :param add_html_url:
    :param edit:
    :param edit_html_url:
    :param delete:
    :param delete_url:
    :param delete_field:
    :param more_delete: 是否开启多条删除
    :param more_delete_url: 多条删除url
    :param more_delete_name: 多条删除name
    :param more_delete_field: 多条删除字段名
    :param action_list: 操作列表
    :param message:
    :return:
    """
    return jinja2_tools.format_template('table/list.html', context={
        'title': title,
        'search_form_item_list': search_form_item_list,
        'data_url': data_url,
        'col_list': col_list,
        'page_name': page_name,
        'limit_name': limit_name,
        'search_params_name': search_params_name,
        'add': add,
        'add_html_url': add_html_url,
        'edit': edit,
        'edit_html_url': edit_html_url,
        'delete': delete,
        'delete_url': delete_url,
        'delete_field': delete_field,
        'more_delete': more_delete,
        'more_delete_url': more_delete_url,
        'more_delete_name': more_delete_name,
        'more_delete_field': more_delete_field,
        'action_list': action_list,
        'message': message
    }, env=template_env)


if __name__ == '__main__':
    print(render_table_template('测试列表', search_form_item_list=[
        LayUiFormItem(title='书名', name='title'),
        LayUiFormItem(title='isbn', name='isbn')
    ], data_url='../api/table.json', col_list=[
        LayUiTableCol(field='id', title='主键'),
        LayUiTableCol(field='city', title='城市'),
        LayUiTableCol(field='sign', title='签名'),
        LayUiTableCol(field='experience', title='积分'),
    ]))



"""
爬虫基类： ：pip install pytest-playwright -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import os
from typing import Callable

from afeng_tools.playwright_tool import http_header_tools, http_cookie_tools
from playwright.sync_api import Page

from afeng_tools.os_tool import os_tools


class BaseSpider:

    def __init__(self, spider_name: str, auth_save_path: str = None,
                 auth_cookie_file_name: str = '.auth_cookie.bin',
                 auth_header_file_name: str = '.auth_header.bin'):
        if auth_save_path is None:
            auth_save_path = os.path.join(os_tools.get_user_home(), f'.{spider_name}')
        os.makedirs(auth_save_path, exist_ok=True)
        self.cookie_file = os.path.join(auth_save_path, auth_cookie_file_name)
        self.header_file = os.path.join(auth_save_path, auth_header_file_name)

    def _refresh_auth(self, web_page: Page, auth_callback: Callable[[Page], None]):
        """刷新认证"""
        print('开始执行认证逻辑')
        http_header_tools.save_headers(web_page, self.header_file)
        auth_callback(web_page)
        print('结束执行认证逻辑')
        print('[保存]认证header')
        http_header_tools.save_headers(web_page, self.header_file)
        print('[保存]认证cookie')
        http_cookie_tools.save_cookies(web_page, self.cookie_file)

    def _load_auth_file(self, web_page: Page, auth_callback: Callable[[Page], None]):
        """加载认证文件"""
        if not os.path.exists(self.cookie_file) or not os.path.exists(self.header_file):
            self._refresh_auth(web_page=web_page, auth_callback=auth_callback)
        return self.header_file, self.cookie_file

    def load_auth(self, web_page: Page, auth_callback: Callable[[Page], None]):
        """
        加载认证
        :param web_page: Page
        :param auth_callback: 认证回调函数，内部实现认证逻辑
        :return:
        """
        print('加载认证信息')
        tmp_header_file, tmp_cookie_file = self._load_auth_file(web_page, auth_callback=auth_callback)
        http_header_tools.set_headers(page=web_page, auth_header_file=tmp_header_file)
        http_cookie_tools.set_cookies(page=web_page, auth_cookie_file=tmp_cookie_file)

    def scroll_to_end(self, web_page: Page, end_locator):
        """滚动到结束"""
        end_more_el = web_page.locator(end_locator)
        while end_more_el.count() <= 0 or end_more_el.is_hidden():
            # 移动鼠标位置
            web_page.mouse.move(20, 100)
            # 向下滚动鼠标
            web_page.mouse.wheel(0, 500)
            web_page.wait_for_timeout(1000)
            end_more_el = web_page.locator(end_locator)


from afeng_tools.weixin_tool.core.format.reply_msg.weixin_format_tool import format_xml_msg
from afeng_tools.weixin_tool.core.format.reply_msg.weixin_models import WeixinReplyMsg, WeixinReplyNewsMsg


def format_msg(reply_msg: WeixinReplyMsg):
    """格式化回复消息"""
    if isinstance(reply_msg, WeixinReplyNewsMsg):
        articles_item_list = list()
        for article in reply_msg.articles:
            articles_item_list.append(format_xml_msg(article))
        setattr(reply_msg, 'articles_str', '\n'.join(articles_item_list))
    return format_xml_msg(reply_msg)






import math
from enum import Enum

from afeng_tools.east_money_tool.quote_info.quote_models import QuoteField


class QuoteFieldEnum(Enum):
    """行情字段枚举"""
    quote_code = QuoteField(name='行情代码', fid_list=['f57'], value_func=lambda x: x['f57'])
    quote_name = QuoteField(name='行情名称', fid_list=['f58'], value_func=lambda x: x['f58'])
    quote_full_name = QuoteField(name='行情全名', fid_list=['f734'], value_func=lambda x: x['f734'])
    trade_timestamp = QuoteField(name='交易时间戳', fid_list=['f86'], value_func=lambda x: x['f86'])
    latest_price = QuoteField(name='最新价', fid_list=['f43', 'f59'],
                              value_func=lambda x: round(x['f43'] / (math.pow(10, x['f59'])), x['f59']))
    up_or_down_amount = QuoteField(name='涨跌额', fid_list=['f169', 'f59'],
                                   value_func=lambda x: round(x['f169'] / (math.pow(10, x['f59'])), x['f59']))
    today_begin_price = QuoteField(name='今开', fid_list=['f46', 'f59'],
                                   value_func=lambda x: round(x['f46'] / (math.pow(10, x['f59'])), x['f59']))
    yesterday_end_price = QuoteField(name='昨收', fid_list=['f60', 'f59'],
                                     value_func=lambda x: round(x['f60'] / (math.pow(10, x['f59'])), x['f59']))
    max_price = QuoteField(name='最高价', fid_list=['f44', 'f59'],
                           value_func=lambda x: round(x['f44'] / (math.pow(10, x['f59'])), x['f59']))
    min_price = QuoteField(name='最低价', fid_list=['f45', 'f59'],
                           value_func=lambda x: round(x['f45'] / (math.pow(10, x['f59'])), x['f59']))
    average_price = QuoteField(name='均价', fid_list=['f71', 'f59'],
                               value_func=lambda x: round(x['f71'] / (math.pow(10, x['f59'])), x['f59']))
    max_end_price = QuoteField(name='涨停价', fid_list=['f51', 'f59'],
                               value_func=lambda x: round(x['f51'] / (math.pow(10, x['f59'])), x['f59']))
    min_end_price = QuoteField(name='跌停价', fid_list=['f52', 'f59'],
                               value_func=lambda x: round(x['f52'] / (math.pow(10, x['f59'])), x['f59']))
    quantity_ratio = QuoteField(name='量比', fid_list=['f50', 'f59'],
                                value_func=lambda x: round(x['f50'] / (math.pow(10, x['f59'])), x['f59']))
    transaction_value = QuoteField(name='成交量', fid_list=['f47'], value_func=lambda x: x['f47'])
    transaction_amount = QuoteField(name='成交额', fid_list=['f48'], value_func=lambda x: x['f48'])
    total_share_capital = QuoteField(name='总股本', fid_list=['f84'], value_func=lambda x: x['f84'])
    negotiable_capital = QuoteField(name='流通股本', fid_list=['f85'], value_func=lambda x: x['f85'])
    per_share_value = QuoteField(name='每股净资产', fid_list=['f92'], value_func=lambda x: x['f92'])
    total_market_value = QuoteField(name='总市值', fid_list=['f116'], value_func=lambda x: x['f116'])
    negotiable_market_value = QuoteField(name='流通市值', fid_list=['f117'], value_func=lambda x: x['f117'])



"""
html内容工具类
"""
import re


def replace_image_tag_delay_src(html_content):
    """转换图片标签的src"""
    if html_content:
        img_tag_pattern = '<img(.*?) src="(.*?)"(.*?)>'
        return re.sub(img_tag_pattern,
                      lambda m: f'<img{m.group(1)} data-src="{m.group(2)}"{m.group(3)}>',
                      html_content)


from typing import Any

from afeng_tools.sqlalchemy_tools.crdu import base_crdu

from afeng_tools.cache_tool.cache_decorator import cache_with_expiration
from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class BlacklistPoService(PoService):
    """
    使用示例：blacklist_po_service = BlacklistPoService(app_info.db_code,BlacklistInfoPo)
    """
    _table_name_ = "tb_blacklist_info"

    def query_by_type_code(self, type_code: str) -> list[Any]:
        return base_crdu.query_all(self.model_type,
                                   self.model_type.type_code == type_code,
                                   db_code=self.db_code)

    @cache_with_expiration(expiration=3600 * 2)
    def query_weixin_black_list(self):
        """查询微信黑名单"""
        data_list = self.query_by_type_code('wx_blacklist')
        return [tmp.type_value for tmp in data_list] if data_list else []

    @cache_with_expiration(expiration=3600 * 24)
    def query_ip_black_list(self):
        """查询ip黑名单"""
        data_list = self.query_by_type_code('ip_blacklist')
        return [tmp.type_value for tmp in data_list] if data_list else []



"""
Linux通配符匹配工具类
"""
import fnmatch


def match_any(string: str, pattern_list: list) -> bool:
    """
    是否匹配linux通配符列表中的任一规则
    :param string: 要进行匹配的内容
    :param pattern_list: linux通配符列表
    :return: True/False
    """
    for pattern in pattern_list:
        if fnmatch.fnmatch(string, pattern):
            return True
    return False


def match_all(string: str, pattern_list: list) -> bool:
    """
    是否匹配linux通配符列表中的所有规则
    :param string: 要进行匹配的内容
    :param pattern_list: linux通配符列表
    :return: True/False
    """
    for pattern in pattern_list:
        if not fnmatch.fnmatch(string, pattern):
            return False
    return True



"""
    xpan

    xpanapi  # noqa: E501

    The version of the OpenAPI document: 0.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from openapi_client.api_client import ApiClient, Endpoint as _Endpoint
from openapi_client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from openapi_client.model.oauth_token_authorization_code_response import OauthTokenAuthorizationCodeResponse
from openapi_client.model.oauth_token_device_code_response import OauthTokenDeviceCodeResponse
from openapi_client.model.oauth_token_device_token_response import OauthTokenDeviceTokenResponse
from openapi_client.model.oauth_token_refresh_token_response import OauthTokenRefreshTokenResponse


class AuthApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.oauth_token_code2token_endpoint = _Endpoint(
            settings={
                'response_type': (OauthTokenAuthorizationCodeResponse,),
                'auth': [],
                'endpoint_path': '/oauth/2.0/token?grant_type=authorization_code&openapi=xpansdk',
                'operation_id': 'oauth_token_code2token',
                'http_method': 'GET',
                'servers': [
                    {
                        'url': "https://openapi.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'code',
                    'client_id',
                    'client_secret',
                    'redirect_uri',
                ],
                'required': [
                    'code',
                    'client_id',
                    'client_secret',
                    'redirect_uri',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'code':
                        (str,),
                    'client_id':
                        (str,),
                    'client_secret':
                        (str,),
                    'redirect_uri':
                        (str,),
                },
                'attribute_map': {
                    'code': 'code',
                    'client_id': 'client_id',
                    'client_secret': 'client_secret',
                    'redirect_uri': 'redirect_uri',
                },
                'location_map': {
                    'code': 'query',
                    'client_id': 'query',
                    'client_secret': 'query',
                    'redirect_uri': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; charset=UTF-8'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.oauth_token_device_code_endpoint = _Endpoint(
            settings={
                'response_type': (OauthTokenDeviceCodeResponse,),
                'auth': [],
                'endpoint_path': '/oauth/2.0/device/code?response_type=device_code&openapi=xpansdk',
                'operation_id': 'oauth_token_device_code',
                'http_method': 'GET',
                'servers': [
                    {
                        'url': "https://openapi.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'client_id',
                    'scope',
                ],
                'required': [
                    'client_id',
                    'scope',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'client_id':
                        (str,),
                    'scope':
                        (str,),
                },
                'attribute_map': {
                    'client_id': 'client_id',
                    'scope': 'scope',
                },
                'location_map': {
                    'client_id': 'query',
                    'scope': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; charset=UTF-8'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.oauth_token_device_token_endpoint = _Endpoint(
            settings={
                'response_type': (OauthTokenDeviceTokenResponse,),
                'auth': [],
                'endpoint_path': '/oauth/2.0/token?grant_type=device_token&openapi=xpansdk',
                'operation_id': 'oauth_token_device_token',
                'http_method': 'GET',
                'servers': [
                    {
                        'url': "https://openapi.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'code',
                    'client_id',
                    'client_secret',
                ],
                'required': [
                    'code',
                    'client_id',
                    'client_secret',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'code':
                        (str,),
                    'client_id':
                        (str,),
                    'client_secret':
                        (str,),
                },
                'attribute_map': {
                    'code': 'code',
                    'client_id': 'client_id',
                    'client_secret': 'client_secret',
                },
                'location_map': {
                    'code': 'query',
                    'client_id': 'query',
                    'client_secret': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; charset=UTF-8'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.oauth_token_refresh_token_endpoint = _Endpoint(
            settings={
                'response_type': (OauthTokenRefreshTokenResponse,),
                'auth': [],
                'endpoint_path': '/oauth/2.0/token?grant_type=refresh_token&openapi=xpansdk',
                'operation_id': 'oauth_token_refresh_token',
                'http_method': 'GET',
                'servers': [
                    {
                        'url': "https://openapi.baidu.com",
                        'description': "No description provided",
                    },
                ]
            },
            params_map={
                'all': [
                    'refresh_token',
                    'client_id',
                    'client_secret',
                ],
                'required': [
                    'refresh_token',
                    'client_id',
                    'client_secret',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'refresh_token':
                        (str,),
                    'client_id':
                        (str,),
                    'client_secret':
                        (str,),
                },
                'attribute_map': {
                    'refresh_token': 'refresh_token',
                    'client_id': 'client_id',
                    'client_secret': 'client_secret',
                },
                'location_map': {
                    'refresh_token': 'query',
                    'client_id': 'query',
                    'client_secret': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json; charset=UTF-8'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

    def oauth_token_code2token(
        self,
        code,
        client_id,
        client_secret,
        redirect_uri,
        **kwargs
    ):
        """oauth_token_code2token  # noqa: E501

        get accesstoken by authorization code  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.oauth_token_code2token(code, client_id, client_secret, redirect_uri, async_req=True)
        >>> result = thread.get()

        Args:
            code (str):
            client_id (str):
            client_secret (str):
            redirect_uri (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            OauthTokenAuthorizationCodeResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['code'] = \
            code
        kwargs['client_id'] = \
            client_id
        kwargs['client_secret'] = \
            client_secret
        kwargs['redirect_uri'] = \
            redirect_uri
        return self.oauth_token_code2token_endpoint.call_with_http_info(**kwargs)

    def oauth_token_device_code(
        self,
        client_id,
        scope,
        **kwargs
    ):
        """oauth_token_device_code  # noqa: E501

        get device code and user code  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.oauth_token_device_code(client_id, scope, async_req=True)
        >>> result = thread.get()

        Args:
            client_id (str):
            scope (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            OauthTokenDeviceCodeResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['client_id'] = \
            client_id
        kwargs['scope'] = \
            scope
        return self.oauth_token_device_code_endpoint.call_with_http_info(**kwargs)

    def oauth_token_device_token(
        self,
        code,
        client_id,
        client_secret,
        **kwargs
    ):
        """oauth_token_device_token  # noqa: E501

        get access_token  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.oauth_token_device_token(code, client_id, client_secret, async_req=True)
        >>> result = thread.get()

        Args:
            code (str):
            client_id (str):
            client_secret (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            OauthTokenDeviceTokenResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['code'] = \
            code
        kwargs['client_id'] = \
            client_id
        kwargs['client_secret'] = \
            client_secret
        return self.oauth_token_device_token_endpoint.call_with_http_info(**kwargs)

    def oauth_token_refresh_token(
        self,
        refresh_token,
        client_id,
        client_secret,
        **kwargs
    ):
        """oauth_token_refresh_token  # noqa: E501

        authorization code  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.oauth_token_refresh_token(refresh_token, client_id, client_secret, async_req=True)
        >>> result = thread.get()

        Args:
            refresh_token (str):
            client_id (str):
            client_secret (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            OauthTokenRefreshTokenResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['refresh_token'] = \
            refresh_token
        kwargs['client_id'] = \
            client_id
        kwargs['client_secret'] = \
            client_secret
        return self.oauth_token_refresh_token_endpoint.call_with_http_info(**kwargs)




from sqlalchemy import Connection, text, select, func

from afeng_tools.sqlalchemy_tool.core.sqlalchemy_session_decorator import auto_conn, auto_commit_conn


@auto_conn()
def query_by_sql(sql, conn: Connection = None):
    """通过sql语句查找"""
    cursor = conn.execute(text(sql))
    return cursor.fetchall()


@auto_commit_conn()
def exec_by_sql(sql, conn: Connection = None):
    """通过sql运行"""
    cursor = conn.execute(text(sql))
    return cursor.lastrowid


def test(model_class, db):
    # select([func.count("*")]).select_from(dataset_no_autoinc)
    select(
        [func.count(model_class.id)], func.length(model_class.name) == 5
    ).execute().first()

    stmt = db.insert().from_select(("data",), select([model_class.data]))



"""
scrapy的selector工具
"""
from typing import Any

from scrapy import Selector
from scrapy.http import Response
from scrapy.selector import SelectorList


def select(selector: Response | Selector, selector_value: str) -> None | Selector | SelectorList | list[Selector]:
    """选择元素"""
    if selector_value is None:
        return None
    if selector_value.startswith('./') or selector_value.startswith('/'):
        return selector.xpath(selector_value)
    else:
        return selector.css(selector_value)


def extract_first(selector: Selector | SelectorList) -> Any:
    """提取第一个元素值"""
    if selector:
        value = selector.extract_first()
        if value and isinstance(value, str):
            return value.strip()
        return value


def select_first_value(selector: Response | Selector, selector_value: str) -> Any:
    """选择第一个元素的值"""
    return extract_first(select(selector, selector_value))



"""
交易日历-工具：
- pip install pandas
- pip install exchange_calendars
"""

from datetime import datetime, timedelta

import exchange_calendars
from pandas import Timestamp

sh_calendar = exchange_calendars.get_calendar("XSHG", side='both')


def next_trade_date(date: str = None) -> str:
    """
    下一个交易日
    :param date: 日期，如：2023-09-12
    :return: 下一个交易日
    """
    if not date:
        trade_time = datetime.now() - timedelta(hours=1)
        timestamp = Timestamp(trade_time.strftime('%Y-%m-%d %H:%M:%S'), tz="Asia/Shanghai")
    else:
        timestamp = Timestamp(f'{date} 23:59:59', tz="Asia/Shanghai")
    return sh_calendar.date_to_session(timestamp, direction="next", _parse=False).strftime("%Y-%m-%d")


def previous_trade_date(date: str = None) -> str:
    """
    上一个交易日
    :param date: 日期，如：2023-09-12
    :return: 上一个交易日
    """
    if not date:
        trade_time = datetime.now() - timedelta(hours=1)
        timestamp = Timestamp(trade_time.strftime('%Y-%m-%d %H:%M:%S'), tz="Asia/Shanghai")
    else:
        timestamp = Timestamp(f'{date} 00:00:00', tz="Asia/Shanghai")
    return sh_calendar.date_to_session(timestamp, direction="previous", _parse=False).strftime("%Y-%m-%d")


if __name__ == '__main__':
    # print(next_trade_date('2023-09-12'))
    # print(previous_trade_date('2023-09-11'))
    # print(previous_trade_date('2023-09-19'))
    print(previous_trade_date())
    print(next_trade_date())

